// ----------------------------------------------------------------------------
// INCLUDES
#include "MemoryManager.hh"
#include "MemoryStrategy.hh"

#include "BitMask.hh"

// ----------------------------------------------------------------------------
// STATIC VARIABLES
BitMask bm;

const MemoryManager::strategymap MemoryManager::strategytable[] = {
    {"bitmask", &bm},
    {0, 0}
};

// ----------------------------------------------------------------------------
// class MemoryManager
MemoryManager::MemoryManager(char* astrategy, caddr_t memadr, size_t memsize) {
    int index = 0;
    while (strategytable[index].name) {
        if (!strcmp(strategytable[index].name, astrategy)) {
            _strategy = strategytable[index].strategy;
            break;
        }
        index++;
    }
    if (!_strategy)
        _strategy = strategytable[0].strategy;
    _strategy->manage(memadr, memsize);
}

MemoryManager::~MemoryManager() {}

void* MemoryManager::malloc(size_t size) {
    if (_strategy) {
        return _strategy->malloc(size);
    }
    return 0;
}

void* MemoryManager::calloc(size_t number, size_t size) {
    if (_strategy) {
        return _strategy->calloc(number, size);
    }
    return 0;
}

void* MemoryManager::realloc(void *ptr, size_t size) {
    if (_strategy) {
        return _strategy->realloc(ptr, size);
    }
    return 0;
}

void MemoryManager::free(void *ptr) {
    if (_strategy) {
        _strategy->free(ptr);
    }
}

void MemoryManager::showBlocks() {
    if (_strategy) {
        _strategy->showBlocks();
    }
}

size_t MemoryManager::size() {
    if (_strategy)
        return _strategy->size();
    return 0;
}
    
size_t MemoryManager::used() {
    if (_strategy)
        return _strategy->used();
    return 0;
}
